from functools import wraps


def func_decorator_without_wraps(func):
    def _func_deco(*args, **kwargs):
        result = func(*args, **kwargs)
        return result

    return _func_deco


def func_decorator_with_wraps(func):
    @wraps(func)
    def _func_deco(*args, **kwargs):
        return func(*args, **kwargs)

    return _func_deco


class ClassDecoratorWithoutWraps:
    def __call__(self, func):
        def decorator(*args, **kwargs):
            return func(*args, **kwargs)

        return decorator


class ClassDecoratorWithWraps:
    def __call__(self, func):
        @wraps(func)
        def decorator(*args, **kwargs):
            return func(*args, **kwargs)

        return decorator



from inspect import isfunction, ismethod
from typing import Callable, Any


def is_valid_function(obj: Any) -> bool:
    return isfunction(obj) or ismethod(obj)


def stripdeco(obj: Any, depth: int = None) -> Callable:
    if not is_valid_function(obj=obj):
        return obj

    if depth == 0:
        return obj

    while True:
        if depth is not None:
            if depth <= 0:
                break
            else:
                depth -= 1

        try:
            obj = obj.__closure__[0].cell_contents
        except TypeError:
            break

    return obj


def run_after_strip(obj: Any, depth: int = None, **kwargs) -> Callable:
    if not is_valid_function(obj=obj) or depth and depth == 0:
        return obj()

    if ismethod(obj):
        self = obj.__self__

    while True:
        if depth is not None:
            if depth <= 0:
                break
            else:
                depth -= 1

        try:
            obj = obj.__closure__[0].cell_contents
        except TypeError:
            break

    try:
        return obj(self, **kwargs)
    except UnboundLocalError:
        return obj(**kwargs)


__all__ = ["stripdeco", "run_after_strip"]



from strip_deco import stripdeco, run_after_strip
from .conftest import (
    ClassDecoratorWithWraps,
    ClassDecoratorWithoutWraps,
    func_decorator_with_wraps,
    func_decorator_without_wraps,
)


def test_stripdeco_mixed_class_with_wrap_and_func_with_wrap():
    @ClassDecoratorWithWraps()
    @func_decorator_with_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_mixed_class_without_wrap_and_func_without_wrap():
    @ClassDecoratorWithoutWraps()
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_mixed_class_with_wrap_and_func_without_wrap():
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_mixed_class_without_wrap_and_func_with_wrap():
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_mixed_class_with_wrap_and_func_with_wrap_depth():
    @ClassDecoratorWithWraps()
    @func_decorator_with_wraps
    @ClassDecoratorWithWraps()
    @func_decorator_with_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test, depth=1)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=2)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=3)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=4)
    assert orig.__closure__ is None


def test_stripdeco_mixed_class_without_wrap_and_func_without_wrap_depth():
    @ClassDecoratorWithoutWraps()
    @func_decorator_without_wraps
    @ClassDecoratorWithoutWraps()
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test, depth=1)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=2)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=3)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=4)
    assert orig.__closure__ is None


def test_stripdeco_mixed_class_with_wrap_and_func_without_wrap_depth():
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test, depth=1)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=2)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=3)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=4)
    assert orig.__closure__ is None


def test_stripdeco_mixed_class_without_wrap_and_func_with_wrap_depth():
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test, depth=1)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=2)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=3)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=4)
    assert orig.__closure__ is None


def test_run_after_strip_mixed_class_with_wrap_and_func_with_wrap():
    @ClassDecoratorWithWraps()
    @func_decorator_with_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_mixed_class_without_wrap_and_func_without_wrap():
    @ClassDecoratorWithoutWraps()
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_mixed_class_with_wrap_and_func_without_wrap():
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_mixed_class_without_wrap_and_func_with_wrap():
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_mixed_class_with_wrap_and_func_with_wrap_depth():
    @ClassDecoratorWithWraps()
    @func_decorator_with_wraps
    @ClassDecoratorWithWraps()
    @func_decorator_with_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test, depth=1)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=2)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=3)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=4)
    assert orig == 1


def test_run_after_strip_mixed_class_without_wrap_and_func_without_wrap_depth():
    @ClassDecoratorWithoutWraps()
    @func_decorator_without_wraps
    @ClassDecoratorWithoutWraps()
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test, depth=1)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=2)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=3)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=4)
    assert orig == 1


def test_run_after_strip_mixed_class_with_wrap_and_func_without_wrap_depth():
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test, depth=1)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=2)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=3)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=4)
    assert orig == 1


def test_run_after_strip_mixed_class_without_wrap_and_func_with_wrap_depth():
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    @ClassDecoratorWithWraps()
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test, depth=1)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=2)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=3)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=4)
    assert orig == 1



from strip_deco import stripdeco, run_after_strip
from .conftest import func_decorator_without_wraps, func_decorator_with_wraps


def test_stripdeco_func_deco_without_wraps():
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_func_deco_with_wraps():
    @func_decorator_with_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_func_deco_with_without_wraps_two_decorator():
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_func_deco_with_wraps_two_decorator():
    @func_decorator_with_wraps
    @func_decorator_with_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_func_deco_with_without_wraps_multiple_decorator():
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test)
    assert orig.__closure__ is None


def test_stripdeco_func_deco_with_without_wraps_depth():
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test, depth=1)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=2)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=3)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=4)
    assert orig.__closure__ is None


def test_stripdeco_func_deco_with_wraps_depth():
    @func_decorator_with_wraps
    @func_decorator_with_wraps
    @func_decorator_with_wraps
    @func_decorator_with_wraps
    def test():
        pass

    assert test.__closure__ is not None
    orig = stripdeco(obj=test, depth=1)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=2)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=3)
    assert orig.__closure__ is not None
    orig = stripdeco(obj=test, depth=4)
    assert orig.__closure__ is None


def test_run_after_strip_func_deco_without_wraps():
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_func_deco_with_wraps():
    @func_decorator_with_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_func_deco_with_without_wraps_two_decorator():
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_func_deco_with_wraps_two_decorator():
    @func_decorator_with_wraps
    @func_decorator_with_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_func_deco_with_without_wraps_multiple_decorator():
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test)
    assert orig == 1


def test_run_after_strip_func_deco_with_without_wraps_depth():
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    @func_decorator_without_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test, depth=1)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=2)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=3)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=4)
    assert orig == 1


def test_run_after_strip_func_deco_with_wraps_depth():
    @func_decorator_with_wraps
    @func_decorator_with_wraps
    @func_decorator_with_wraps
    @func_decorator_with_wraps
    def test():
        return 1

    assert test.__closure__ is not None
    orig = run_after_strip(obj=test, depth=1)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=2)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=3)
    assert orig == 1
    orig = run_after_strip(obj=test, depth=4)
    assert orig == 1



# encoding: utf-8
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="strip-deco",
    version="0.0.5",
    author="Hide",
    author_email="padocon@naver.com",
    description="Easily strip decorator from function or class method",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/teamhide/strip-deco",
    packages=setuptools.find_packages(),
    tests_require=["pytest"],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.4",
)
